(* ------------------------------------------------------------------ *)
(* Prime Factors Kata                                                 *)
(* Version: 1.00                                                      *)
(* Author: Peter Kofler, www.code-cop.org                             *)
(* System: Turbo Pascal 6.01                                          *)
(* ------------------------------------------------------------------ *)
PROGRAM PrimeF_T;

USES TPUnit, Lists, PrimeFac;

TYPE PrimeFactorsTest = OBJECT(TestCase)
        _act:ArrayListPtr;
        PROCEDURE SetUp; VIRTUAL;
        PROCEDURE TearDown; VIRTUAL;
        PROCEDURE TestOne;
        PROCEDURE AssertPrime(prime:Longint);
        PROCEDURE TestTwo;
        PROCEDURE TestThree;
        PROCEDURE AssertProduct(one, two:Longint);
        PROCEDURE TestFour;
        PROCEDURE TestSix;
        PROCEDURE TestEight;
        PROCEDURE TestNine;
        PROCEDURE TestMax;
        PROCEDURE Run; VIRTUAL;
     END;

PROCEDURE PrimeFactorsTest.SetUp;
BEGIN
   _act := NIL;
END;

PROCEDURE PrimeFactorsTest.TearDown;
BEGIN
   IF _act <> NIL THEN dispose(_act, Destroy);
END;

PROCEDURE PrimeFactorsTest.TestOne;
BEGIN
   _act := PrimeFactors.generate(1);
   assertNotNil('generate(1) rerturn value', _act);
   assertEquals('generate(1) number factors', 0, _act^.size);
END;

PROCEDURE PrimeFactorsTest.AssertPrime(prime:Longint);
VAR s:String;
BEGIN
   _act := PrimeFactors.generate(prime);
   Str(prime, s);
   assertEquals('generate('+s+') number factors', 1, _act^.size);
   assertEquals('generate('+s+') first factor', prime, _act^.get(0));
END;

PROCEDURE PrimeFactorsTest.TestTwo;
BEGIN
   AssertPrime(2);
END;

PROCEDURE PrimeFactorsTest.TestThree;
BEGIN
   AssertPrime(3);
END;

PROCEDURE PrimeFactorsTest.AssertProduct(one, two:Longint);
VAR s:String;
BEGIN
   _act := PrimeFactors.generate(one*two);
   Str(one * two, s);
   assertEquals('generate('+s+') number factors', 2, _act^.size);
   assertEquals('generate('+s+') first factor', one, _act^.get(0));
   assertEquals('generate('+s+') second factor', two, _act^.get(1));
END;

PROCEDURE PrimeFactorsTest.TestFour;
BEGIN
   AssertProduct(2, 2);
END;

PROCEDURE PrimeFactorsTest.TestSix;
BEGIN
   AssertProduct(2, 3);
END;

PROCEDURE PrimeFactorsTest.TestEight;
BEGIN
   _act := PrimeFactors.generate(8);
   assertEquals('generate(8) number factors', 3, _act^.size);
   assertEquals('generate(8) first factor', 2, _act^.get(0));
   assertEquals('generate(8) second factor', 2, _act^.get(1));
   assertEquals('generate(8) third factor', 2, _act^.get(2));
END;

PROCEDURE PrimeFactorsTest.TestNine;
BEGIN
   AssertProduct(3, 3);
END;

PROCEDURE PrimeFactorsTest.TestMax;
BEGIN
   AssertPrime(2147483647);
END;

PROCEDURE PrimeFactorsTest.Run;
BEGIN
   RunTest('TestOne', @PrimeFactorsTest.TestOne );
   RunTest('TestTwo', @PrimeFactorsTest.TestTwo );
   RunTest('TestThree', @PrimeFactorsTest.TestThree );
   RunTest('TestFour', @PrimeFactorsTest.TestFour );
   RunTest('TestSix', @PrimeFactorsTest.TestSix );
   RunTest('TestEight', @PrimeFactorsTest.TestEight );
   RunTest('TestNine', @PrimeFactorsTest.TestNine );
   RunTest('TestMax', @PrimeFactorsTest.TestMax );
END;

CONST testee:PrimeFactorsTest = ();
BEGIN
   testee.Run
END.
