{F+}
(* ------------------------------------------------------------------ *)
(* Minimalist xUnit implementation for Turbo Pascal similar to JUnit. *)
(* Version: 1.10                                                      *)
(* Author: Peter Kofler, www.code-cop.org                             *)
(* System: Turbo Pascal 6.01                                          *)
(* ------------------------------------------------------------------ *)
UNIT TPUnit;

INTERFACE
{
  Test cases must subclass TestCase:

  USES TPUnit;

  TYPE ExampleTest = OBJECT(TestCase)
          PROCEDURE TestMethod;
          PROCEDURE Run; VIRTUAL;
       END;

  Tests are added as methods to the class as usual
  and use asserts provided by the superclass. The
  first failed assertion stops program execution.

  PROCEDURE ExampleTest.TestMethod;
  BEGIN
     AssertEquals('use asserts in tests', 2, 1+1);
  END;

  Due to the lack of introspection each test has to
  be called manually by implementing the Run method.
  SetUp and TearDown are called by RunTest as expected.

  PROCEDURE ExampleTest.Run;
  BEGIN
     RunTest('TestMethod', @ExampleTest.TestMethod);
  END;

  There is no testrunner. To execute the test case
  Run is called on an instance of the test.

  CONST testee:ExampleTest = ();
  BEGIN
     testee.Run
  END.
}

TYPE TestCase = OBJECT
        { Assert }
        PROCEDURE AssertNil(message:String; actual:Pointer);
        PROCEDURE AssertNotNil(message:String; actual:Pointer);
        PROCEDURE AssertEquals(message:String; expected, actual:Longint);
        PROCEDURE AssertEqualsStr(message:String; expected, actual:String);
        PROCEDURE AssertTrue(message:String; actual:Boolean);
        PROCEDURE AssertFalse(message:String; actual:Boolean);
        PROCEDURE Fail(message:String);
        { Test }
        PROCEDURE RunTest(name:String; testMethod:Pointer);
        PROCEDURE Run; VIRTUAL;
        { TestCase }
        PROCEDURE SetUp; VIRTUAL;
        PROCEDURE TearDown; VIRTUAL;
     END;

IMPLEMENTATION

USES Crt;

PROCEDURE TestCase.AssertEquals(message:String; expected, actual:Longint);
VAR ex, ac:String;
BEGIN
   IF expected <> actual THEN
   BEGIN
      Str(expected, ex);
      Str(actual, ac);
      Fail(Concat(message, ' expected ', ex, ' but was ', ac));
   END;
END;

PROCEDURE TestCase.AssertEqualsStr(message:String; expected, actual:String);
BEGIN
   IF expected <> actual THEN
   BEGIN
      Fail(Concat(message, ' expected ', expected, ' but was ', actual));
   END;
END;

PROCEDURE TestCase.AssertNotNil(message:String; actual:Pointer);
BEGIN
   AssertFalse(message, actual = NIL);
END;

PROCEDURE TestCase.AssertNil(message:String; actual:Pointer);
BEGIN
   AssertTrue(message, actual = NIL);
END;

PROCEDURE TestCase.AssertTrue(message:String; actual:Boolean);
BEGIN
   IF NOT actual THEN
   BEGIN
      Fail(message);
   END;
END;

PROCEDURE TestCase.AssertFalse(message:String; actual:Boolean);
BEGIN
   AssertTrue(message, NOT actual);
END;

PROCEDURE TestCase.Fail(message:String);
BEGIN
   TearDown;

   TextColor(Red);
   Writeln(' - FAILED');
   NormVideo;
   Writeln(message);

   Halt(1);
END;

PROCEDURE CallClassPtr(pt:Pointer; VAR cls:TestCase);
{ call the procedure on the test case }
VAR s,o:Word;
BEGIN
   s := Seg(cls);
   o := Ofs(cls);
   ASM
      mov DI, [o]
      mov AX, [s]
      push AX
      push DI
      call [pt.dword]
   END;
END;

PROCEDURE TestCase.RunTest(name:String; testMethod:Pointer);
BEGIN
   Write('TEST ', name);
   SetUp;
   CallClassPtr(testMethod, self);
   TearDown;

   TextColor(Green);
   Writeln(' - OK');
   NormVideo;
END;

PROCEDURE TestCase.Run;
{ not implemented - needs to be filled in subclass }
BEGIN
   RunError; 
END;

PROCEDURE TestCase.SetUp;
BEGIN
END;

PROCEDURE TestCase.TearDown;
BEGIN
END;

BEGIN
   Crt.ClrScr;
END.
